"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.zSchemaOptions = void 0;
exports.ZSchema = require("z-schema");
// @todo 264263 - utilize the schema validator in sp-build-common
const fs = __importStar(require("fs"));
const os_1 = require("os");
const path = __importStar(require("path"));
// Discover the schema files
const schemasDirectory = path.join(__dirname, 'manifestSchemas', 'jsonSchemas');
const schemaFiles = fs.readdirSync(schemasDirectory);
const schemas = schemaFiles.map((schemaFile) => require(path.join(schemasDirectory, schemaFile)));
const manifestSchema = require('./manifestSchemas/jsonSchemas/client-side-component-manifest.schema.json');
const applicationSchema = require('./manifestSchemas/jsonSchemas/client-side-application-manifest.schema.json');
const prefabAppSchema = require('./manifestSchemas/jsonSchemas/prefab-app-manifest.schema.json');
const webPartSchema = require('./manifestSchemas/jsonSchemas/client-side-web-part-manifest.schema.json');
const commandSetSchema = require('./manifestSchemas/jsonSchemas/command-set-extension-manifest.schema.json');
const extensionSchema = require('./manifestSchemas/jsonSchemas/client-side-extension-manifest.schema.json');
const librarySchema = require('./manifestSchemas/jsonSchemas/client-side-library-manifest.schema.json');
const assemblySchema = require('./manifestSchemas/jsonSchemas/client-side-assembly-manifest.schema.json');
const multiVersionSchema = require('./manifestSchemas/jsonSchemas/client-side-multi-version-manifest.schema.json');
// local copy of site scripts schema and draft-07 schema
const siteScriptsSchema = require('./manifestSchemas/remote/site-design-script-actions.schema.json');
const draft04Schema = require('./manifestSchemas/remote/draft-04.schema.json');
const SITE_SCRIPTS_SCHEMA_URL = 'https://developer.microsoft.com/json-schemas/sp/site-design-script-actions.schema.json';
const DRAFT_07_SCHEMA_URL = 'http://json-schema.org/draft-07/schema#';
const siteScriptsSchemaPromise = fetch(SITE_SCRIPTS_SCHEMA_URL)
    .then((response) => {
    return response.text();
})
    .catch((error) => {
    // eslint-disable-next-line no-console
    console.error(`Failed to fetch remote schema: ${error}`);
    // fallback to local copy
    return JSON.stringify(siteScriptsSchema);
});
exports.zSchemaOptions = {
    breakOnFirstError: true,
    forceAdditional: true,
    forceItems: true,
    forceMaxLength: false,
    forceProperties: false,
    noExtraKeywords: true,
    noTypeless: true,
    noEmptyStrings: true
};
const schemaValidator = new exports.ZSchema(exports.zSchemaOptions);
const draft07ValidationOptions = {
    strictMode: false,
    ignoreUnresolvableReferences: true,
    ignoreUnknownFormats: true
};
//
// this one is needed for the site scripts reference.
// We're faking draft-07 schema with draft-04 schema as z-schema doesn't support draft-07
//
// eslint-disable-next-line @typescript-eslint/no-explicit-any
schemaValidator.setRemoteReference(DRAFT_07_SCHEMA_URL, draft04Schema, draft07ValidationOptions);
siteScriptsSchemaPromise
    .then((remoteSchema) => {
    // ZSchema typedef doesn't contain setRemoteReference method. But it does exist in the implementation and referenced in the documentation.
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    schemaValidator.setRemoteReference(SITE_SCRIPTS_SCHEMA_URL, JSON.parse(remoteSchema), {
        strictMode: false,
        ignoreUnresolvableReferences: true
    });
    schemaValidator.validateSchema(schemas);
})
    .catch((error) => {
    // swallowing the error here
});
class ManifestValidator {
    static registerRemoteSchemaReferences() {
        return siteScriptsSchemaPromise
            .then(() => {
            // no-op;
        })
            .catch(() => {
            // no-op;
        });
    }
    static validateApplicationManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, applicationSchema);
    }
    static validatePrefabAppManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, prefabAppSchema);
    }
    static validateCommandSetManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, commandSetSchema);
    }
    static validateExtensionManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, extensionSchema);
    }
    static validateWebPartManifest(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    manifest) {
        return ManifestValidator._validateManifest(manifest, webPartSchema);
    }
    static validateLibraryManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, librarySchema);
    }
    static validateAssemblyManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, assemblySchema);
    }
    static validateMultiVersionManifest(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    manifest) {
        return ManifestValidator._validateManifest(manifest, multiVersionSchema);
    }
    static validateManifest(manifest) {
        return ManifestValidator._validateManifest(manifest, manifestSchema);
    }
    static extractInnerErrorMessages(errors) {
        const printZSchemaError = (error) => {
            let innerErrors = [];
            (error.inner || []).forEach((innerErr) => {
                innerErrors = innerErrors.concat(printZSchemaError(innerErr));
            });
            return [`(${error.path}) ${error.message}`].concat(innerErrors);
        };
        let errorList = [];
        errors.map((error) => {
            errorList = errorList.concat(printZSchemaError(error));
        });
        return errorList;
    }
    static getFormattedErrorMessage(errors) {
        return this.extractInnerErrorMessages(errors).join(os_1.EOL);
    }
    static _validateManifest(
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    manifest, schema) {
        if (!schema) {
            throw new Error('Unable to find the manifest schema.');
        }
        if (typeof manifest === 'string') {
            manifest = JSON.parse(manifest);
        }
        const result = schemaValidator.validate(manifest, schema);
        return {
            result: result,
            errors: schemaValidator.getLastErrors() || []
        };
    }
}
exports.default = ManifestValidator;
//# sourceMappingURL=manifestSchemaValidator.js.map