import type { IClientSideComponentManifest, ILocalizedString } from './IClientSideComponentManifest';
/**
 * This is the manifest for a PREFAB application.
 *
 * @alpha
 */
export interface IPrefabAppManifest extends IClientSideComponentManifest {
    /**
     * Title of the application as a dictionary of locale keys to title values. This value will be
     *  displayed to the user in the (page creation) interface.
     *
     * @remarks
     *
     * Supported values: a dictionary of locale keys to strings. Should always have a 'default' key.
     *
     * Example: `"My Application"`
     *
     * ```
     *          {
     *            "default": "My Application"
     *            "en-us": "My Application",
     *            "fr-fr": "Ma demande",
     *            "zh": "我的應用程式"
     *          }
     * ```
     */
    title: ILocalizedString;
    /**
     * Description of the application represented as a dictionary of locale keys to description values.
     *  This value will be displayed to the user in the (page creation) interface.
     *
     * @remarks
     *
     * Supported values: a dictionary of locale keys to strings. Should always have a 'default' key.
     *
     * Examples:
     *
     * ```
     * "An application for searching for things."
     * ```
     * or
     * ```
     *          {
     *            "default": "An application for searching for things.",
     *            "en-us": "An application for searching for things.",
     *            "fr-fr": "Une demande pour la recherche de choses.",
     *            "zh": "申請尋找的東西。"
     *          }
     * ```
     */
    description: ILocalizedString;
    /**
     * Specifies if there is a FCE (Finish Configuration Experience) available for the app. True if no FCE is available.
     */
    isConfigured: boolean;
    /**
     * Describes how the PREFAB app can be installed: NewSite or ExistingSite. If omitted the app supports both.
     */
    appInstallationTypes?: Array<'NewSite' | 'ExistingSite'>;
    /**
     * Site script actions.
     *
     * @remarks - See https://developer.microsoft.com/json-schemas/sp/site-design-script-actions.schema.json
     */
    actions?: ISiteScriptActionBase[];
}
/**
 * Base definition for Site Script action.
 *
 * @alpha
 */
export interface ISiteScriptActionBase {
    verb: string;
    [key: string]: any;
}
//# sourceMappingURL=IPrefabAppManifest.d.ts.map